@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * Mon Aug  8 20:35:30 BST 1994  Wilf. (G.Wilford@ee.surrey.ac.uk)
  */
@@ -31,6 +31,12 @@
 #include <stdio.h>
 #include <errno.h>
 
+#if defined(STDC_HEADERS) || defined(HAVE_STRING_H)
+#include <string.h>
+#elif defined(HAVE_STRINGS_H)
+#include <strings.h>
+#endif /* STDC_HEADERS */
+
 #if HAVE_SYS_FILE_H
 #  include <sys/file.h> /* for flock() */
 #endif
@@ -66,24 +72,24 @@ struct hashtable *loop_check_hash;
 
 void test_insert (int line, const datum key, const datum cont)
 {
-	fprintf (stderr, "(%d) key: \"%s\", cont: \"%.40s\"\n", line, key.dptr,
-		 cont.dptr);
+	debug ("(%d) key: \"%s\", cont: \"%.40s\"\n",
+	       line, MYDBM_DPTR (key), MYDBM_DPTR (cont));
 }
 #else /* !FAST_BTREE */
 #define B_FLAGS		0	/* do not allow any duplicate keys */
 #endif /* FAST_BTREE */
 
 /* release the lock and close the database */
-int btree_close (DB *dbf)
+int btree_close (DB *db)
 {
-	(void) flock ((dbf->fd) (dbf), LOCK_UN);
-	return (dbf->close) (dbf);
+	(void) flock ((db->fd) (db), LOCK_UN);
+	return (db->close) (db);
 }
 
 /* open a btree type database, with file locking. */
 DB *btree_flopen (char *filename, int flags, int mode)
 {
-	DB *dbf;
+	DB *db;
 	BTREEINFO b;
 	int lock_op;
 	int lock_failed;
@@ -127,59 +133,60 @@ DB *btree_flopen (char *filename, int flags, int mode)
 		/* opening the db is destructive, need to lock first */
 		int fd;
 
-		dbf = NULL;
+		db = NULL;
 		lock_failed = 1;
 		fd = open (filename, flags & ~O_TRUNC, mode);
 		if (fd != -1) {
 			if (!(lock_failed = flock (fd, lock_op)))
-				dbf = dbopen (filename, flags, mode,
-					      DB_BTREE, &b);
+				db = dbopen (filename, flags, mode,
+					     DB_BTREE, &b);
 			close (fd);
 		}
 	} else {
-		dbf = dbopen (filename, flags, mode, DB_BTREE, &b);
-		if (dbf)
-			lock_failed = flock ((dbf->fd) (dbf), lock_op);
+		db = dbopen (filename, flags, mode, DB_BTREE, &b);
+		if (db)
+			lock_failed = flock ((db->fd) (db), lock_op);
 	}
 
-	if (!dbf)
+	if (!db)
 		return NULL;
 
 	if (lock_failed) {
 		gripe_lock (filename);
-		btree_close (dbf);
+		btree_close (db);
 		return NULL;
 	}
 
-	return dbf;
+	return db;
 }
 
 /* do a replace when we have the duplicate flag set on the database -
    we must do a del and insert, as a direct insert will not wipe out the
    old entry */
-int btree_replace (DB *dbf, datum key, datum cont)
+int btree_replace (DB *db, datum key, datum cont)
 {
 #ifdef FAST_BTREE
 	test_insert (__LINE__, key, cont);
-	return (dbf->put) (dbf, (DBT *) &key, (DBT *) &cont, R_CURSOR);
+	return (db->put) (db, (DBT *) &key, (DBT *) &cont, R_CURSOR);
 #else /* normal BTREE */
-	return (dbf->put) (dbf, (DBT *) &key, (DBT *) &cont, 0);
+	return (db->put) (db, (DBT *) &key, (DBT *) &cont, 0);
 #endif /* FAST_BTREE */
 }
 
-int btree_insert (DB *dbf, datum key, datum cont)
+int btree_insert (DB *db, datum key, datum cont)
 {
-	return (dbf->put) (dbf, (DBT *) &key, (DBT *) &cont, R_NOOVERWRITE);
+	return (db->put) (db, (DBT *) &key, (DBT *) &cont, R_NOOVERWRITE);
 }
 
 /* generic fetch routine for the btree database */
-datum btree_fetch (DB *dbf, datum key)
+datum btree_fetch (DB *db, datum key)
 {
 	datum data;
 
-	if ((dbf->get) (dbf, (DBT *) &key, (DBT *) &data, 0)) {
-		data.dptr = NULL;
-		data.dsize = 0;
+	memset (&data, 0, sizeof data);
+
+	if ((db->get) (db, (DBT *) &key, (DBT *) &data, 0)) {
+		memset (&data, 0, sizeof data);
 		return data;
 	}
 
@@ -187,17 +194,20 @@ datum btree_fetch (DB *dbf, datum key)
 }
 
 /* return 1 if the key exists, 0 otherwise */
-int btree_exists (DB *dbf, datum key)
+int btree_exists (DB *db, datum key)
 {
 	datum data;
-	return ((dbf->get) (dbf, (DBT *) &key, (DBT *) &data, 0) ? 0 : 1);
+	return ((db->get) (db, (DBT *) &key, (DBT *) &data, 0) ? 0 : 1);
 }
 
 /* initiate a sequential access */
-static __inline__ datum btree_findkey (DB *dbf, u_int flags)
+static __inline__ datum btree_findkey (DB *db, u_int flags)
 {
 	datum key, data;
 
+	memset (&key, 0, sizeof key);
+	memset (&data, 0, sizeof data);
+
 	if (flags == R_FIRST) {
 		if (loop_check_hash) {
 			hash_free (loop_check_hash);
@@ -207,52 +217,50 @@ static __inline__ datum btree_findkey (DB *dbf, u_int flags)
 	if (!loop_check_hash)
 		loop_check_hash = hash_create (&plain_hash_free);
 
-	if (((dbf->seq) (dbf, (DBT *) &key, (DBT *) &data, flags))) {
-		key.dptr = NULL;
-		key.dsize = 0;
+	if (((db->seq) (db, (DBT *) &key, (DBT *) &data, flags))) {
+		memset (&key, 0, sizeof key);
 		return key;
 	}
 
-	if (hash_lookup (loop_check_hash, key.dptr, key.dsize)) {
+	if (hash_lookup (loop_check_hash,
+	                 MYDBM_DPTR (key), MYDBM_DSIZE (key))) {
 		/* We've seen this key already, which is broken. Return NULL
 		 * so the caller doesn't go round in circles.
 		 */
-		if (debug)
-			fprintf (stderr, "Corrupt database! Already seen %*s. "
-					 "Attempting to recover ...\n",
-				 (int) key.dsize, key.dptr);
-		key.dptr = NULL;
-		key.dsize = 0;
+		debug ("Corrupt database! Already seen %*s. "
+		       "Attempting to recover ...\n",
+		       (int) MYDBM_DSIZE (key), MYDBM_DPTR (key));
+		memset (&key, 0, sizeof key);
 		return key;
 	}
 
-	hash_install (loop_check_hash, key.dptr, key.dsize, NULL);
+	hash_install (loop_check_hash, MYDBM_DPTR (key), MYDBM_DSIZE (key),
+	              NULL);
 
 	return copy_datum (key);
 }
 
 /* return the first key in the db */
-datum btree_firstkey (DB *dbf)
+datum btree_firstkey (DB *db)
 {
-	return btree_findkey (dbf, R_FIRST);
+	return btree_findkey (db, R_FIRST);
 }
 
 /* return the next key in the db. NB. This routine only works if the cursor
    has been previously set by btree_firstkey() since it was last opened. So
    if we close/reopen a db mid search, we have to manually set up the
    cursor again. */
-datum btree_nextkey (DB *dbf)
+datum btree_nextkey (DB *db)
 {
-	return btree_findkey (dbf, R_NEXT);
+	return btree_findkey (db, R_NEXT);
 }
 
 /* compound nextkey routine, initialising key and content */
-int btree_nextkeydata (DB *dbf, datum *key, datum *cont)
+int btree_nextkeydata (DB *db, datum *key, datum *cont)
 {
 	int status;
 
-	if ((status = (dbf->seq) (dbf, (DBT *) key, (DBT *) cont,
-				  R_NEXT)) != 0)
+	if ((status = (db->seq) (db, (DBT *) key, (DBT *) cont, R_NEXT)) != 0)
 		return status;
 
 	*key = copy_datum (*key);
@@ -279,15 +287,17 @@ int dbstore (struct mandata *in, char *basename)
 	datum key, cont;
 	int status;
 
- 	key.dsize = strlen (basename) + 1;
+	memset (&key, 0, sizeof key);
+	memset (&cont, 0, sizeof cont);
 
- 	if (key.dsize == 1) {
- 		if (debug)
- 			dbprintf (in);
+ 	MYDBM_DSIZE (key) = strlen (basename) + 1;
+
+ 	if (MYDBM_DSIZE (key) == 1) {
+		dbprintf (in);
  		return 2;
  	}
 
-	key.dptr = basename;
+	MYDBM_SET_DPTR (key, basename);
 
 	/* initialise the cursor to (possibly) our key/cont */
 	status = (dbf->seq) (dbf, (DBT *) &key, (DBT *) &cont, R_CURSOR);
@@ -296,13 +306,13 @@ int dbstore (struct mandata *in, char *basename)
 		gripe_get (__LINE__);
 
 	/* either nothing was found or the key was not an exact match */
-	else if (status == 1 || !STREQ (key.dptr, basename)) {
+	else if (status == 1 || !STREQ (MYDBM_DPTR (key), basename)) {
 		cont = make_content (in);
-		key.dptr = basename;
-		key.dsize = strlen (basename) + 1;
+		MYDBM_SET_DPTR (key, basename);
+		MYDBM_DSIZE (key) = strlen (basename) + 1;
 		test_insert (__LINE__, key, cont);
 		status = (dbf->put) (dbf, (DBT *) &key, (DBT *) &cont, 0);
-		free (cont.dptr);
+		free (MYDBM_DPTR (cont));
 
 	/* There is already a key with this name */
 	} else {
@@ -311,27 +321,28 @@ int dbstore (struct mandata *in, char *basename)
 		while (1) {
 			struct mandata old;
 
+			/* TODO: what if cont is unset? */
 			cont = copy_datum (cont);
-			split_content (cont.dptr, &old);
+			split_content (MYDBM_DPTR (cont), &old);
 			if (STREQ (in->ext, old.ext)) {
 				cont = make_content (in);
 				status = replace_if_necessary (in, &old,
 							       key, cont);
-				free (cont.dptr);
+				free (MYDBM_DPTR (cont));
 				free_mandata_elements (&old);
 				break;
 			}
 			free_mandata_elements (&old);
 			status = (dbf->seq) (dbf, (DBT *) &key, (DBT *) &cont,
 					     R_NEXT);
-			if (!STREQ (key.dptr, basename)) {
-				key.dptr = basename;
-				key.dsize = strlen (basename) + 1;
+			if (!STREQ (MYDBM_DPTR (key), basename)) {
+				MYDBM_SET_DPTR (key, basename);
+				MYDBM_DSIZE (key) = strlen (basename) + 1;
 				cont = make_content (in);
 				test_insert (__LINE__, key, cont);
 				status = (dbf->put) (dbf, (DBT *) &key,
 						     (DBT *) &cont, 0);
-				free (cont.dptr);
+				free (MYDBM_DPTR (cont));
 				break;
 			}
 		}
@@ -347,14 +358,17 @@ static struct mandata *dblookup (char *page, char *section, int flags)
 	datum key, cont;
 	int status;
 
-	key.dptr = page;
-	key.dsize = strlen (page) + 1;
+	memset (&key, 0, sizeof key);
+	memset (&cont, 0, sizeof cont);
+
+	MYDBM_SET_DPTR (key, page);
+	MYDBM_DSIZE (key) = strlen (page) + 1;
 
 	/* initialise the cursor to (possibly) our key/cont */
 	status = (dbf->seq) (dbf, (DBT *) &key, (DBT *) &cont, R_CURSOR);
 
 	/* either nothing was found or the key was not an exact match */
-	if (status == 1 || !STREQ (page, key.dptr))
+	if (status == 1 || !STREQ (page, MYDBM_DPTR (key)))
 		return NULL;
 	if (status == -1)
 		gripe_get (__LINE__);
@@ -363,8 +377,9 @@ static struct mandata *dblookup (char *page, char *section, int flags)
 	null_me = &(info->next);
 
 	do {
+		/* TODO: what if cont is unset? */
 		cont = copy_datum (cont);
-		split_content (cont.dptr, info);
+		split_content (MYDBM_DPTR (cont), info);
 
 		if (!(section == NULL ||
 		    STRNEQ (section, info->ext,
@@ -383,7 +398,7 @@ static struct mandata *dblookup (char *page, char *section, int flags)
 			gripe_get (__LINE__);
 
 		/* run out of identical keys */
-	} while (!(status == 1 || !STREQ (page, key.dptr)));
+	} while (!(status == 1 || !STREQ (page, MYDBM_DPTR (key))));
 
 	free (info);
 	*null_me = NULL;
