@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with man-db; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * Mon Mar 13 20:27:36 GMT 1995  Wilf. (G.Wilford@ee.surrey.ac.uk) 
  */
@@ -27,9 +27,6 @@
 #  include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-/* Need _GNU_SOURCE for FNM_CASEFOLD. */
-#define _GNU_SOURCE
-
 #include <stdio.h>
 
 #if defined(STDC_HEADERS)
@@ -63,6 +60,7 @@ extern char *strrchr();
 #include "manconfig.h"
 #include "lib/error.h"
 #include "lib/hashtable.h"
+#include "lib/cleanup.h"
 #include "globbing.h"
 
 const char *extension;
@@ -70,7 +68,7 @@ static const char *mandir_layout = MANDIR_LAYOUT;
 
 #ifdef TEST
 
-#  include <libintl.h>
+#  include "lib/gettext.h"
 #  define _(String) gettext (String)
 
 #  ifdef HAVE_GETOPT_H
@@ -80,7 +78,6 @@ static const char *mandir_layout = MANDIR_LAYOUT;
 #  endif /* HAVE_GETOPT_H */
 
 char *program_name;
-int debug = 0;
 
 static const struct option long_options[] =
 {
@@ -126,17 +123,18 @@ static int parse_layout (const char *layout)
 		for (layoutp = upper_layout; *layoutp; layoutp++)
 			*layoutp = CTYPE (toupper, *layoutp);
 
-		if (strstr (layout, "GNU"))
+		if (strstr (upper_layout, "GNU"))
 			flags |= LAYOUT_GNU;
-		if (strstr (layout, "HPUX"))
+		if (strstr (upper_layout, "HPUX"))
 			flags |= LAYOUT_HPUX;
-		if (strstr (layout, "IRIX"))
+		if (strstr (upper_layout, "IRIX"))
 			flags |= LAYOUT_IRIX;
-		if (strstr (layout, "SOLARIS"))
+		if (strstr (upper_layout, "SOLARIS"))
 			flags |= LAYOUT_SOLARIS;
-		if (strstr (layout, "BSD"))
+		if (strstr (upper_layout, "BSD"))
 			flags |= LAYOUT_BSD;
 
+		free (upper_layout);
 		return flags;
 	}
 }
@@ -172,26 +170,23 @@ static struct dirent_hashent *update_directory_cache (const char *path)
 	DIR *dir;
 	struct dirent *entry;
 
-	if (!dirent_hash)
+	if (!dirent_hash) {
 		dirent_hash = hash_create (&dirent_hash_free);
+		push_cleanup ((cleanup_fun) hash_free, dirent_hash, 0);
+	}
 	cache = hash_lookup (dirent_hash, path, strlen (path));
 
 	/* Check whether we've got this one already. */
 	if (cache) {
-		if (debug)
-			fprintf (stderr, "update_directory_cache %s: hit\n",
-				 path);
+		debug ("update_directory_cache %s: hit\n", path);
 		return cache;
 	}
 
-	if (debug)
-		fprintf (stderr, "update_directory_cache %s: miss\n", path);
+	debug ("update_directory_cache %s: miss\n", path);
 
 	dir = opendir (path);
 	if (!dir) {
-		if (debug)
-			fprintf (stderr, "can't open directory %s: %s\n",
-				 path, strerror (errno));
+		debug_error ("can't open directory %s", path);
 		return NULL;
 	}
 
@@ -242,20 +237,22 @@ static int match_in_directory (const char *path, const char *pattern,
 	char **bsearched;
 	size_t i;
 
+	/* look_for_file declares this static, so it's zero-initialised.
+	 * globfree() can deal with checking it before freeing.
+	 */
+	globfree (pglob);
+
 	pglob->gl_pathc = 0;
 	pglob->gl_pathv = NULL;
 	pglob->gl_offs = 0;
 
 	cache = update_directory_cache (path);
 	if (!cache) {
-		if (debug)
-			fprintf (stderr, "directory cache update failed\n");
+		debug ("directory cache update failed\n");
 		return -1;
 	}
 
-	if (debug)
-		fprintf (stderr, "globbing pattern in %s: %s\n",
-			 path, pattern);
+	debug ("globbing pattern in %s: %s\n", path, pattern);
 
 	pglob->gl_pathv = xmalloc (allocated * sizeof (char *));
 	flags = ignore_case ? FNM_CASEFOLD : 0;
@@ -266,6 +263,7 @@ static int match_in_directory (const char *path, const char *pattern,
 	bsearched = bsearch (&pattern_start, cache->names, cache->names_len,
 			     sizeof *cache->names, &pattern_compare);
 	if (!bsearched) {
+		free (pattern_start.pattern);
 		pglob->gl_pathv[0] = NULL;
 		return 0;
 	}
@@ -285,9 +283,7 @@ static int match_in_directory (const char *path, const char *pattern,
 		if (fnm)
 			continue;
 
-		if (debug)
-			fprintf (stderr, "matched: %s/%s\n",
-				 path, cache->names[i]);
+		debug ("matched: %s/%s\n", path, cache->names[i]);
 
 		if (pglob->gl_pathc >= allocated) {
 			allocated *= 2;
@@ -310,31 +306,30 @@ static int match_in_directory (const char *path, const char *pattern,
 	return 0;
 }
 
-char **look_for_file (const char *unesc_hier, const char *sec,
+char **look_for_file (const char *hier, const char *sec,
 		      const char *unesc_name, int cat, int match_case)
 {
 	char *pattern = NULL, *path = NULL;
 	static glob_t gbuf;
+	static int cleanup_installed = 0;
 	int status = 1;
 	static int layout = -1;
-	char *hier, *name;
+	char *name;
 
-	/* As static struct is allocated and contains NULLs we don't need 
-	   to check it before attempting a free. Let globfree() do that */
-
-	globfree (&gbuf);
+	if (!cleanup_installed) {
+		/* appease valgrind */
+		push_cleanup ((cleanup_fun) globfree, &gbuf, 0);
+		cleanup_installed = 1;
+	}
 
 	/* This routine only does a minimum amount of matching. It does not
 	   find cat files in the alternate cat directory. */
 
 	if (layout == -1) {
 		layout = parse_layout (mandir_layout);
-		if (debug)
-			fprintf (stderr, "Layout is %s (%d)\n",
-				 mandir_layout, layout);
+		debug ("Layout is %s (%d)\n", mandir_layout, layout);
 	}
 
-	hier = escape_shell (unesc_hier);
 	name = escape_shell (unesc_name);
 
 	/* allow lookups like "3x foo" to match "../man3/foo.3x" */
@@ -428,7 +423,6 @@ char **look_for_file (const char *unesc_hier, const char *sec,
 	}
 
 	free (name);
-	free (hier);
 	free (path);
 	free (pattern);
 
@@ -456,17 +450,17 @@ static void usage (int status)
 
 int main (int argc, char **argv)
 {
-	int c, option_index;
+	int c;
 	int i;
 	int match_case = 0;
 
 	program_name = xstrdup (basename (argv[0]));
 
 	while ((c = getopt_long (argc, argv, args,
-				 long_options, &option_index)) != -1) {
+				 long_options, NULL)) != -1) {
 		switch (c) {
 			case 'd':
-				debug = 1;
+				debug_level = 1;
 				break;
 			case 'e':
 				extension = optarg;
