@@ -101,8 +101,7 @@ int is_changed (const char *fa, const char *fb)
 	int fb_stat;
 	int status = 0;
 
-	if (debug)
-		fprintf (stderr, "is_changed: a=%s, b=%s", fa, fb);
+	debug ("is_changed: a=%s, b=%s", fa, fb);
 
 	fa_stat = stat (fa, &fa_sb);
 	if (fa_stat != 0)
@@ -113,8 +112,7 @@ int is_changed (const char *fa, const char *fb)
 		status |= 2;
 
 	if (status != 0) {
-		if (debug)
-			fprintf (stderr, " (%d)\n", -status);
+		debug (" (%d)\n", -status);
 		return -status;
 	}
 
@@ -126,8 +124,7 @@ int is_changed (const char *fa, const char *fb)
 
 	status |= (fa_sb.st_mtime != fb_sb.st_mtime);
 
-	if (debug)
-		fprintf (stderr, " (%d)\n", status);
+	debug (" (%d)\n", status);
 	return status;
 }
 
@@ -168,7 +165,7 @@ char *escape_shell (const char *unesc)
 		if ((*unescp >= '0' && *unescp <= '9') ||
 		    (*unescp >= 'A' && *unescp <= 'Z') ||
 		    (*unescp >= 'a' && *unescp <= 'z') ||
-		    strchr (",-./:=@_", *unescp))
+		    strchr (",-./:@_", *unescp))
 			*escp++ = *unescp;
 		else {
 			*escp++ = '\\';
@@ -223,3 +220,40 @@ char *trim_spaces (const char *s)
 		--length;
 	return xstrndup (s, length);
 }
+
+char *lang_dir (const char *filename)
+{
+	char *ld;	/* the lang dir: point to static data */
+	const char *fm;	/* the first "/man/" dir */
+	const char *sm;	/* the second "/man?/" dir */
+
+	ld = xstrdup ("");
+	if (!filename) 
+		return ld;
+
+	/* Check whether filename is in a man page hierarchy. */
+	fm = strstr (filename, "/man/");
+	if (!fm)
+		return ld;
+	sm = strstr (fm + 3, "/man");
+	if (!sm)
+		return ld;
+	if (sm[5] != '/')
+		return ld;
+	if (!strchr ("123456789lno", sm[4]))
+		return ld;
+
+	/* If there's no lang dir element, it's an English man page. */
+	if (sm == fm + 4)
+		return xstrdup ("C");
+
+	/* found a lang dir */
+	fm += 5;
+	sm = strchr (fm, '/');
+	if (!sm)
+		return ld;
+	free (ld);
+	ld = xstrndup (fm, sm - fm);
+	debug ("found lang dir element %s\n", ld);
+	return ld;
+}
